package jp.co.yamaha_motor.xe.xe0.xe01f061_02.batch;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import jp.co.yamaha_motor.commons.core.util.NumberUtil;
import jp.co.yamaha_motor.commons.yfc.batch.util.BatchUtils;
import jp.co.yamaha_motor.commons.yfc.batch.util.LogReportWriter;
import jp.co.yamaha_motor.commons.yfc.batch.util.ThreadLogReportWriter;
import jp.co.yamaha_motor.commons.yfc.utility.util.DateUtil;
import jp.co.yamaha_motor.commons.yfc.utility.util.StringUtil;
import jp.co.yamaha_motor.l4commons.batch.core.PymacBatchTemplate;
import jp.co.yamaha_motor.xe.xe0.xe01f061_02.model.Xe01f061_02Parametermodel;
import jp.co.yamaha_motor.xe.xe0.xe01f061_02.model.Xe01f061_02SuppStockMapModel;
import jp.co.yamaha_motor.xe.xe0.xe01f061_02.model.Xe01f061_02SuppStockModel;
import org.apache.commons.lang.StringUtils;

public class Xe01f061_02 extends PymacBatchTemplate {

   protected Xe01f061_02Parametermodel getParameterModelInstance() throws Exception {
      return new Xe01f061_02Parametermodel();
   }

   protected void setParameter(Xe01f061_02Parametermodel pm, String name, String value) throws Exception {
      if("arg0".equals(name)) {
         pm.setSysOwnerCd(value);
      }

   }

   protected void prepareParameterAfter(Xe01f061_02Parametermodel pm) throws Exception {
      LogReportWriter writer = ThreadLogReportWriter.getWriter();
      writer.writeLog("", " [Argument Value]", (String)null, (String)null, (Throwable)null);
      writer.writeLog("", " sysOwnerCd   : " + pm.getSysOwnerCd(), (String)null, (String)null, (Throwable)null);
   }

   protected void init(Xe01f061_02Parametermodel pm) throws Exception {
   }

   protected void main(Xe01f061_02Parametermodel pm) throws Exception {
      List suppStockModelList = this.getSuppStockModelList(pm);
      List suppStockModelListWithQty = this.getSuppStockModelListWithQty(suppStockModelList, pm);
      this.updateSuppStock(suppStockModelListWithQty, pm);
   }

   protected void term(Xe01f061_02Parametermodel pm) throws Exception {
   }

   private List getSuppStockModelList(Xe01f061_02Parametermodel pm) throws Exception {
      ArrayList suppStockModelList = new ArrayList();
      String sysDate = pm.getSysDate8();
      String sql = "";
      sql = sql + " SELECT sys_owner_cd_   AS  sysOwnerCd   ";
      sql = sql + "       ,parts_no        AS  partsNo      ";
      sql = sql + "       ,supplier_code   AS  supplierCode ";
      sql = sql + "       ,user_code       AS  userCode     ";
      sql = sql + "       ,onway_update    AS  onwayUpdate  ";
      sql = sql + "   FROM supp_stock                       ";
      sql = sql + "  WHERE sys_owner_cd_ = ?                ";
      Object[] params = new Object[]{pm.getSysOwnerCd()};
      ResultSet resultSet = this.dao.getResult(sql, params);

      try {
         while(BatchUtils.hasNext(resultSet)) {
            Xe01f061_02SuppStockModel suppStockModel = (Xe01f061_02SuppStockModel)BatchUtils.convertResultModel(resultSet, Xe01f061_02SuppStockModel.class);
            String onwayUpdateYmd = DateUtil.toStringFormatGMTTime(suppStockModel.getOnwayUpdate(), "yyyyMMdd");
            if(StringUtil.lt(onwayUpdateYmd, sysDate)) {
               suppStockModelList.add(suppStockModel);
            }
         }
      } finally {
         resultSet.close();
      }

      return suppStockModelList;
   }

   private List getSuppStockModelListWithQty(List suppStockModelList, Xe01f061_02Parametermodel pm) throws Exception {
      ArrayList suppStockModelListWithQty = new ArrayList();
      HashMap map = new HashMap();
      Timestamp minOnwayUpdate = null;
      BigDecimal qty = BigDecimal.ZERO;
      String sysDate = pm.getSysDate8();
      String sysOwnerCd = pm.getSysOwnerCd();
      minOnwayUpdate = this.getMinOnWayUpdate(sysOwnerCd);
      String minOnwayUpdateYMD = DateUtil.toStringFormatGMTTime(minOnwayUpdate, "yyyyMMdd");
      Map map1 = this.getLdTrnTransMonthMap(minOnwayUpdate, sysOwnerCd, map);
      if(!StringUtils.equals(this.getYmFromYmdDate(sysOwnerCd, sysDate), this.getYmFromYmdDate(sysOwnerCd, minOnwayUpdateYMD))) {
         map1 = this.getLdTrnTransMonthAndTransMap(minOnwayUpdate, sysDate, sysOwnerCd, map1, minOnwayUpdateYMD);
      }

      Iterator i$ = suppStockModelList.iterator();

      while(i$.hasNext()) {
         Xe01f061_02SuppStockModel suppStockModel = (Xe01f061_02SuppStockModel)i$.next();
         Xe01f061_02SuppStockModel suppStockModelWithQty = new Xe01f061_02SuppStockModel();
         suppStockModelWithQty.setSysOwnerCd(suppStockModel.getSysOwnerCd());
         suppStockModelWithQty.setPartsNo(suppStockModel.getPartsNo());
         suppStockModelWithQty.setSupplierCode(suppStockModel.getSupplierCode());
         suppStockModelWithQty.setUserCode(suppStockModel.getUserCode());
         String QtyKey = suppStockModel.getSysOwnerCd() + suppStockModel.getPartsNo() + suppStockModel.getSupplierCode() + suppStockModel.getUserCode();
         qty = (BigDecimal)map1.get(QtyKey);
         if(qty == null) {
            qty = new BigDecimal(0);
         }

         suppStockModelWithQty.setInQty(qty);
         suppStockModelListWithQty.add(suppStockModelWithQty);
      }

      return suppStockModelListWithQty;
   }

   private Map getLdTrnTransMonthAndTransMap(Timestamp minOnwayUpdate, String sysDate, String sysOwnerCd, Map map, String minOnwayUpdateYMD) {
      String yestoday = DateUtil.addDate(-1, sysDate);
      String onwayUpdate = minOnwayUpdateYMD == null?yestoday:minOnwayUpdateYMD;
      String beginYm = this.getYmFromYmdDate(sysOwnerCd, onwayUpdate);
      String endYm = this.getYmFromYmdDate(sysOwnerCd, sysDate);
      String month = "";

      while(StringUtil.lt(beginYm, endYm)) {
         month = StringUtils.substring(beginYm, 4, 6);
         String sql1 = "";
         sql1 = sql1 + " SELECT COALESCE(sum(input_qty),0) AS inQty    ";
         sql1 = sql1 + "   FROM ld_trn_trans_" + month;
         sql1 = sql1 + "  WHERE sys_owner_cd = ?                       ";
         sql1 = sql1 + "    AND input_operation_id = \'LD14\'            ";
         sql1 = sql1 + "    AND inter_txn = \'21\'                       ";
         sql1 = sql1 + "    AND CAST(create_datetime AS TEXT) > ?      ";
         sql1 = sql1 + "  GROUP BY sys_owner_cd,itemno,supplier,usercd ";
         Object[] param1 = new Object[]{sysOwnerCd, minOnwayUpdate};
         ResultSet resultSet = this.dao.getResult(sql1, param1);
         String sql2 = "";
         sql2 = sql2 + " SELECT COALESCE(sum(-input_qty),0) AS inQty   ";
         sql2 = sql2 + "   FROM ld_trn_trans_" + month;
         sql2 = sql2 + "  WHERE sys_owner_cd = ?                       ";
         sql2 = sql2 + "    AND input_operation_id = \'LD14\'            ";
         sql2 = sql2 + "    AND inter_txn = \'41\'                       ";
         sql2 = sql2 + "    AND CAST(create_datetime AS TEXT) > ?      ";
         sql2 = sql2 + "  GROUP BY sys_owner_cd,itemno,supplier,usercd ";
         Object[] param2 = new Object[]{sysOwnerCd, minOnwayUpdate};
         ResultSet resultSet2 = this.dao.getResult(sql2, param2);

         while(true) {
            try {
               Xe01f061_02SuppStockMapModel e;
               String modelKey;
               BigDecimal modelValue;
               BigDecimal monthValue;
               BigDecimal finallyQty;
               if(BatchUtils.hasNext(resultSet)) {
                  e = (Xe01f061_02SuppStockMapModel)BatchUtils.convertResultModel(resultSet, Xe01f061_02SuppStockMapModel.class);
                  modelKey = e.getSysOwnerCd() + e.getItemno() + e.getSupplier() + e.getUsercd();
                  modelValue = e.getInQty();
                  monthValue = (BigDecimal)map.get(modelKey);
                  new BigDecimal(0);
                  if(modelKey != null && modelValue != null) {
                     if(monthValue != null) {
                        finallyQty = NumberUtil.addBigDecimal(monthValue, modelValue);
                     } else {
                        finallyQty = monthValue;
                     }

                     map.put(modelKey, finallyQty);
                  }
                  continue;
               }

               while(BatchUtils.hasNext(resultSet2)) {
                  e = (Xe01f061_02SuppStockMapModel)BatchUtils.convertResultModel(resultSet2, Xe01f061_02SuppStockMapModel.class);
                  modelKey = e.getSysOwnerCd() + e.getItemno() + e.getSupplier() + e.getUsercd();
                  modelValue = e.getInQty();
                  monthValue = (BigDecimal)map.get(modelKey);
                  new BigDecimal(0);
                  if(modelKey != null && modelValue != null) {
                     if(monthValue != null) {
                        finallyQty = NumberUtil.addBigDecimal(monthValue, modelValue);
                     } else {
                        finallyQty = monthValue;
                     }

                     map.put(modelKey, finallyQty);
                  }
               }
            } catch (Exception var22) {
               var22.printStackTrace();
            }

            beginYm = DateUtil.addMonthFromYM(beginYm, 1);
            break;
         }
      }

      return map;
   }

   private Map getLdTrnTransMonthMap(Timestamp minOnwayUpdate, String sysOwnerCd, Map map) {
      String sql1 = "";
      sql1 = sql1 + " SELECT COALESCE(sum(input_qty),0) AS inQty    ";
      sql1 = sql1 + "       ,sys_owner_cd                           ";
      sql1 = sql1 + "       ,itemno                                 ";
      sql1 = sql1 + "       ,supplier                               ";
      sql1 = sql1 + "       ,usercd                                 ";
      sql1 = sql1 + "   FROM ld_trn_trans_month                     ";
      sql1 = sql1 + "  WHERE sys_owner_cd = ?                       ";
      sql1 = sql1 + "    AND input_operation_id = \'LD14\'            ";
      sql1 = sql1 + "    AND inter_txn = \'21\'                       ";
      sql1 = sql1 + "    AND CAST(create_datetime AS TEXT) > ?      ";
      sql1 = sql1 + "  GROUP BY sys_owner_cd,itemno,supplier,usercd ";
      Object[] param1 = new Object[]{sysOwnerCd, minOnwayUpdate};
      ResultSet resultSet = this.dao.getResult(sql1, param1);
      String sql2 = "";
      sql2 = sql2 + " SELECT COALESCE(sum(-input_qty),0) AS inQty   ";
      sql2 = sql2 + "       ,sys_owner_cd                           ";
      sql2 = sql2 + "       ,itemno                                 ";
      sql2 = sql2 + "       ,supplier                               ";
      sql2 = sql2 + "       ,usercd                                 ";
      sql2 = sql2 + "   FROM ld_trn_trans_month                     ";
      sql2 = sql2 + "  WHERE sys_owner_cd = ?                       ";
      sql2 = sql2 + "    AND input_operation_id = \'LD14\'            ";
      sql2 = sql2 + "    AND inter_txn = \'41\'                       ";
      sql2 = sql2 + "    AND CAST(create_datetime AS TEXT) > ?      ";
      sql2 = sql2 + "  GROUP BY sys_owner_cd,itemno,supplier,usercd ";
      Object[] param2 = new Object[]{sysOwnerCd, minOnwayUpdate};
      ResultSet resultSet2 = this.dao.getResult(sql2, param2);

      try {
         Xe01f061_02SuppStockMapModel e;
         BigDecimal modelValue;
         String modelKey;
         while(BatchUtils.hasNext(resultSet)) {
            e = (Xe01f061_02SuppStockMapModel)BatchUtils.convertResultModel(resultSet, Xe01f061_02SuppStockMapModel.class);
            new BigDecimal(0);
            modelKey = null;
            modelKey = e.getSysOwnerCd() + e.getItemno() + e.getSupplier() + e.getUsercd();
            modelValue = e.getInQty();
            if(modelKey != null && modelValue != null) {
               map.put(modelKey, modelValue);
            }
         }

         while(BatchUtils.hasNext(resultSet2)) {
            e = (Xe01f061_02SuppStockMapModel)BatchUtils.convertResultModel(resultSet2, Xe01f061_02SuppStockMapModel.class);
            new BigDecimal(0);
            modelKey = null;
            modelKey = e.getSysOwnerCd() + e.getItemno() + e.getSupplier() + e.getUsercd();
            modelValue = e.getInQty();
            if(modelKey != null && modelValue != null) {
               BigDecimal value1 = (BigDecimal)map.get(modelKey);
               if(value1 != null) {
                  map.put(modelKey, NumberUtil.addBigDecimal(modelValue, value1));
               } else {
                  map.put(modelKey, modelValue);
               }
            }
         }
      } catch (Exception var14) {
         var14.printStackTrace();
      }

      return map;
   }

   private Timestamp getMinOnWayUpdate(String sysOwnerCd) {
      Timestamp minOnwayUpdateYmd = null;
      String sql = "";
      sql = sql + " SELECT MIN(onway_update) AS  minOnwayUpdate ";
      sql = sql + "   FROM supp_stock                           ";
      sql = sql + "  WHERE sys_owner_cd_ = ?                    ";
      Object[] params = new Object[]{sysOwnerCd};
      ResultSet resultSet = this.dao.getResult(sql, params);

      try {
         while(BatchUtils.hasNext(resultSet)) {
            Xe01f061_02SuppStockModel e = (Xe01f061_02SuppStockModel)BatchUtils.convertResultModel(resultSet, Xe01f061_02SuppStockModel.class);
            minOnwayUpdateYmd = e.getMinOnwayUpdate();
         }
      } catch (Exception var14) {
         var14.printStackTrace();
      } finally {
         try {
            resultSet.close();
         } catch (SQLException var13) {
            var13.printStackTrace();
         }

      }

      return minOnwayUpdateYmd;
   }

   private String getYmFromYmdDate(String sysOwnerCd, String date) {
      String sql = "";
      sql = sql + "   SELECT object_month               ";
      sql = sql + "     FROM ld_month_proc_cont         ";
      sql = sql + "    WHERE sys_owner_cd = ?  ";
      sql = sql + "      AND date_begin  <= ?  ";
      sql = sql + "      AND date_end    >= ?  ";
      Object[] param = new Object[]{sysOwnerCd, date, date};
      Object[] result = this.dao.getRecord(sql, param);
      return StringUtil.toString(result[0]);
   }

   private void updateSuppStock(List suppStockModelListWithQty, Xe01f061_02Parametermodel pm) throws Exception {
      Iterator i$ = suppStockModelListWithQty.iterator();

      while(i$.hasNext()) {
         Xe01f061_02SuppStockModel suppStockModelWithQty = (Xe01f061_02SuppStockModel)i$.next();
         String sql = "";
         sql = sql + " UPDATE supp_stock                         ";
         sql = sql + "    SET onway_qty     = (onway_qty-?)      ";
         sql = sql + "       ,upd_count     = upd_count + 1      ";
         sql = sql + "       ,upd_person    = ?                  ";
         sql = sql + "       ,upd_time      = ?                  ";
         sql = sql + "       ,onway_update  = ?                  ";
         sql = sql + "  WHERE sys_owner_cd_ = ?                  ";
         sql = sql + "    AND parts_no      = ?                  ";
         sql = sql + "    AND supplier_code = ?                  ";
         sql = sql + "    AND user_code     = ?                  ";
         Object[] params = new Object[]{suppStockModelWithQty.getInQty(), pm.getUserId(), pm.getSysDatetime(), pm.getSysDatetime(), suppStockModelWithQty.getSysOwnerCd(), suppStockModelWithQty.getPartsNo(), suppStockModelWithQty.getSupplierCode(), suppStockModelWithQty.getUserCode()};
         this.dao.setRecord(sql, params);
      }

   }
}
